import asyncio
import hashlib
import os

from dotenv import load_dotenv
from elpis import mcp_servers

if os.getenv('ELPIS_ENV_FILE'):
    load_dotenv(os.getenv('ELPIS_ENV_FILE'))
else:
    load_dotenv()

from elpis import constants, i18n, tools
from elpis.langgraph_agent import ElpisMem0Agent

lang = os.getenv('LANG')
if not lang:
    lang = 'en'

print(constants.BANNER, flush=True)

lang = i18n.select_lang(lang)
print(lang.WELCOME_INFO, flush=True)

# initialize codebase
if os.getenv('EMBEDDING_MODEL_KEY_PREFIX'):
    tools.init_codebase(os.getcwd())

print(f"[{constants.AI_AGENT_NAME}] Using LangGraph implementation", flush=True)
# Generate session_id based on current directory path MD5
current_dir = os.getcwd()
session_id = hashlib.md5(current_dir.encode('utf-8')).hexdigest()

mcp_file_path = os.getenv('MCP_FILE_PATH', default=os.path.join(os.getcwd(), 'mcp.json'))
mcp_tools = None
if os.path.exists(mcp_file_path):
    mcp_tools = asyncio.run(mcp_servers.get_mcp_tools(mcp_file_path))

agent = ElpisMem0Agent(session_id=session_id, lang=lang,
                       mcp_tools=mcp_tools)

graph = agent.graph



import json

from langchain_core.tools import BaseTool
from langchain_mcp_adapters.client import MultiServerMCPClient
from pydantic import BaseModel


class McpServer(BaseModel):
    command: str
    args: list[str]
    transport: str = 'stdio'


async def get_mcp_tools(mcp_file_path: str) -> list[BaseTool] | None:
    with open(mcp_file_path, 'r') as f:
        mcp_config = f.read()
    try:
        mcp_servers = json.loads(mcp_config)['mcpServers']
        # 参数校验与预处理
        mcp_servers = {name: McpServer(**config).model_dump() for name, config in mcp_servers.items()}
        mcp_client = MultiServerMCPClient(mcp_servers)
        return await mcp_client.get_tools()
    except Exception as e:
        return None



import os
from typing import Optional, Dict, Any


def new_mem0(model_prefix_key: str,
             embedding_model_prefix_key: str = None):
    from mem0.memory.main import AsyncMemory as _AsyncMemory, logger
    from mem0.configs.base import MemoryConfig
    from pydantic import ValidationError

    class AsyncMemory(_AsyncMemory):

        @classmethod
        def from_config(cls, config_dict: dict):
            try:
                c = cls._process_config(config_dict)
                c = MemoryConfig(**config_dict)
            except ValidationError as e:
                logger.error(f"Configuration validation error: {e}")
                raise
            return cls(c)

        async def search(
                self,
                query: str,
                *,
                user_id: Optional[str] = None,
                agent_id: Optional[str] = None,
                run_id: Optional[str] = None,
                limit: int = 100,
                filters: Optional[Dict[str, Any]] = None,
                threshold: Optional[float] = None,
        ):
            ans = await super().search(
                query,
                user_id=user_id,
                agent_id=agent_id,
                run_id=run_id,
                limit=limit,
                filters=filters,
                threshold=threshold,
            )
            if ans and 'results' in ans:
                return ans['results']
            return ans


    model_provider = os.getenv(f"{model_prefix_key}_MODEL_PROVIDER", default='openai')

    config = {
        "llm": {
            "provider": model_provider,
            "config": {
                "model": os.getenv(f"{model_prefix_key}_MODEL"),
                f"{model_provider}_base_url": os.getenv(f"{model_prefix_key}_BASE_URL"),
                "api_key": os.getenv(f"{model_prefix_key}_API_KEY"),
                "temperature": float(os.getenv(f"{model_prefix_key}_TEMPERATURE", default="0.3")),
                "max_tokens": int(os.getenv(f"{model_prefix_key}_MAX_TOKENS", default="4096")),
            }
        },
        "vector_store": {
            "provider": "faiss",
            "config": {
                "path": os.path.join(os.path.join(os.getcwd(), '.elpis'), 'faiss_mem0'),
                "embedding_model_dims": int(os.getenv('MEM0_VECTOR_STORE_EMBEDDING_MODEL_DIMS', default=1536)),
            }
        }
    }

    if embedding_model_prefix_key:
        embedding_model_provider = os.getenv(f"{embedding_model_prefix_key}_MODEL_PROVIDER", default='openai')
        config["embedder"] = {
            'provider': embedding_model_provider,
            'config': {
                'model': os.getenv(f"{embedding_model_prefix_key}_MODEL"),
                f'{embedding_model_provider}_base_url': os.getenv(f"{embedding_model_prefix_key}_BASE_URL",
                                                                  default='https://api.openai.com/v1'),
                "api_key": os.getenv(f"{embedding_model_prefix_key}_API_KEY"),
            }
        }

    return AsyncMemory.from_config(config)


def new_mem0_client(mem0_api_key: str):
    from mem0 import AsyncMemoryClient
    import httpx
    return AsyncMemoryClient(api_key=mem0_api_key)
